@@ -54,9 +54,10 @@ static bool print_mode = NORMAL;
 static bool standard_baudrate = true;
 static void (*print)(char c);
 static int fd;
-static bool map_inlcrnl = false;
-static bool map_onlcrnl = false;
-static bool map_odelbs = false;
+static bool map_i_nl_crnl = false;
+static bool map_o_cr_nl = false;
+static bool map_o_nl_crnl = false;
+static bool map_o_del_bs = false;
 
 #define tio_printf(format, args...) \
 { \
@@ -100,6 +101,7 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 tio_printf(" ctrl-t ?   List available key commands");
                 tio_printf(" ctrl-t b   Send break");
                 tio_printf(" ctrl-t c   Show configuration");
+                tio_printf(" ctrl-t e   Toggle local echo mode");
                 tio_printf(" ctrl-t h   Toggle hexadecimal mode");
                 tio_printf(" ctrl-t l   Clear screen");
                 tio_printf(" ctrl-t q   Quit");
@@ -119,6 +121,7 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                 tio_printf(" Flow: %s", option.flow);
                 tio_printf(" Stopbits: %d", option.stopbits);
                 tio_printf(" Parity: %s", option.parity);
+                tio_printf(" Local Echo: %s", option.local_echo ? "yes":"no");
                 tio_printf(" Output delay: %d", option.output_delay);
                 if (option.map[0] != 0)
                     tio_printf(" Map flags: %s", option.map);
@@ -126,6 +129,10 @@ void handle_command_sequence(char input_char, char previous_char, char *output_c
                     tio_printf(" Log file: %s", option.log_filename);
                 break;
 
+            case KEY_E:
+                option.local_echo = !option.local_echo;
+                break;
+
             case KEY_H:
                 /* Toggle hexadecimal printing mode */
                 if (print_mode == NORMAL)
@@ -415,13 +422,13 @@ void tty_configure(void)
             else if (strcmp(token,"ICRNL") == 0)
                 tio.c_iflag |= ICRNL;
             else if (strcmp(token,"OCRNL") == 0)
-                tio.c_oflag |= OCRNL;
+                map_o_cr_nl = true;
             else if (strcmp(token,"ODELBS") == 0)
-                map_odelbs = true;
+                map_o_del_bs = true;
             else if (strcmp(token,"INLCRNL") == 0)
-                map_inlcrnl = true;
+                map_i_nl_crnl = true;
             else if (strcmp(token, "ONLCRNL") == 0)
-                map_onlcrnl = true;
+                map_o_nl_crnl = true;
             else
             {
                 printf("Error: Unknown mapping flag %s\n", token);
@@ -511,6 +518,15 @@ void tty_restore(void)
         tty_disconnect();
 }
 
+static void optional_local_echo(char c)
+{
+    if (!option.local_echo)
+        return;
+    print(c);
+    if (option.log)
+        log_write(c);
+}
+
 int tty_connect(void)
 {
     fd_set rdfs;           /* Read file descriptor set */
@@ -603,7 +619,7 @@ int tty_connect(void)
                     rx_total++;
 
                     /* Map input character */
-                    if ((input_char == '\n') && (map_inlcrnl))
+                    if ((input_char == '\n') && (map_i_nl_crnl))
                     {
                         print('\r');
                         print('\n');
@@ -650,13 +666,16 @@ int tty_connect(void)
                 if (forward)
                 {
                     /* Map output character */
-                    if ((output_char == 127) && (map_odelbs))
+                    if ((output_char == 127) && (map_o_del_bs))
                         output_char = '\b';
+                    if ((output_char == '\r') && (map_o_cr_nl))
+                        output_char = '\n';
 
                     /* Map newline character */
-                    if ((output_char == '\n') && (map_onlcrnl)) {
+                    if ((output_char == '\n') && (map_o_nl_crnl)) {
                         char r = '\r';
 
+                        optional_local_echo(r);
                         status = write(fd, &r, 1);
                         if (status < 0)
                             warning_printf("Could not write to tty device");
@@ -666,6 +685,7 @@ int tty_connect(void)
                     }
 
                     /* Send output to tty device */
+                    optional_local_echo(output_char);
                     status = write(fd, &output_char, 1);
                     if (status < 0)
                         warning_printf("Could not write to tty device");
